package com.xmlparser;

import android.util.Log;
import android.util.Xml;

import com.model.Coo;
import com.model.NFe;
import com.model.NumeroSerieAgente;
import com.model.Response;

import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;

public class NDDParser {

    private static final String PKG = "NDDParser";
    private final String NAMESPACE = null;

    private static final String ROOT_TAG = "nfce";
    private static final String RESPONSE_TAG = "response";
    private static final String AUTH_TAG = "authorization";
    private static final String COO_TAG = "consultaNumeroCoo";
    private static final String STATUS_TAG = "consultaStatus";

    public List parse(InputStream in) throws XmlPullParserException, IOException, ParseException {
        try {
            XmlPullParser parser = Xml.newPullParser();
            parser.setFeature(XmlPullParser.FEATURE_PROCESS_NAMESPACES, false);
            parser.setInput(in, null);
            parser.nextTag();

            return read(parser);
        } finally {
            in.close();
        }
    }

    public List read(XmlPullParser parser) throws XmlPullParserException, IOException, ParseException {
        List ret = new ArrayList<>();

        parser.require(XmlPullParser.START_TAG, NAMESPACE, ROOT_TAG);
        while (parser.next() != XmlPullParser.END_TAG) {

            if (parser.getEventType() != XmlPullParser.START_TAG)
                continue;

            String name = parser.getName();

            switch (name) {
                case RESPONSE_TAG:
                    ret.add(processResponse(parser));
                    break;
                case AUTH_TAG:
                    ret.add(processAuth(parser));
                    break;
                case COO_TAG:
                    ret.add(processCOO(parser));
                    break;
                case STATUS_TAG:
                    ret.add(processNumeroSerieAgente(parser));
                    break;
                default:
                    Log.i(PKG, "Unexpected tag : " + name);
                    skip(parser);
                    break;
            }
        }

        return ret;
    }


    private Response processResponse(XmlPullParser parser) throws XmlPullParserException, IOException, ParseException {

        parser.require(XmlPullParser.START_TAG, NAMESPACE, RESPONSE_TAG);

        Integer code = null;
        String desc = null;

        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.getEventType() != XmlPullParser.START_TAG)
                continue;

            String name = parser.getName();

            switch (name) {
                case "code":
                    code = Integer.parseInt(readText(parser));
                    break;
                case "description":
                    desc = readText(parser);
                    break;
                default:
                    throw new IllegalArgumentException("Unexpected field: " + name);
            }

        }
        return new Response(code, desc);
    }

    private Coo processCOO(XmlPullParser parser) throws XmlPullParserException, IOException {
        parser.require(XmlPullParser.START_TAG, NAMESPACE, COO_TAG);

        Integer code = null;

        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.getEventType() != XmlPullParser.START_TAG)
                continue;

            String name = parser.getName();

            switch (name) {
                case "coonumber":
                    code = Integer.parseInt(readText(parser));
                    break;
                default:
                    throw new IllegalArgumentException("Unexpected field: " + name);
            }
        }
        return new Coo(code);
    }

    private NumeroSerieAgente processNumeroSerieAgente(XmlPullParser parser) throws XmlPullParserException, IOException {
        parser.require(XmlPullParser.START_TAG, NAMESPACE, STATUS_TAG);

        while (parser.next() != XmlPullParser.END_TAG) {
            if(parser.getEventType() != XmlPullParser.START_TAG)
                continue;

            String name = parser.getName();
            
            switch (name) {
              case "NSERIE":
                  return new NumeroSerieAgente(readText(parser));
              default:
                  skip(parser);
            }
        }
        return null;
    }


    private NFe processAuth(XmlPullParser parser) throws XmlPullParserException, IOException {

        parser.require(XmlPullParser.START_TAG, NAMESPACE, AUTH_TAG);
        String accesskey = null;
        String digVal = null;
        String nProt = null;
        String recebimento = null;

        while (parser.next() != XmlPullParser.END_TAG) {
            if (parser.getEventType() != XmlPullParser.START_TAG)
                continue;
            Log.d(PKG, "HERE I AM WITH TAG: " + parser.getName());
            switch (parser.getName()) {
                case "accesskey":
                    accesskey = readText(parser);
                    break;
                case "digval":
                    digVal = readText(parser);
                    break;
                case "nprot":
                    nProt = readText(parser);
                    break;
                case "dhrecbto":
                    recebimento = readText(parser);
                    break;
                default:
                    throw new IllegalArgumentException("Invalid tagname: " + parser.getName());
            }
        }

        return new NFe(accesskey, digVal, nProt, recebimento);
    }

    private void skip(XmlPullParser parser) throws XmlPullParserException, IOException {
        if (parser.getEventType() != XmlPullParser.START_TAG) {
            throw new IllegalStateException();
        }
        int depth = 1;
        while (depth != 0) {
            switch (parser.next()) {
                case XmlPullParser.END_TAG:
                    depth--;
                    break;
                case XmlPullParser.START_TAG:
                    depth++;
                    break;
            }
        }
    }

    private String readText(XmlPullParser parser) throws XmlPullParserException, IOException {
        String ret = "";
        if (parser.next() == XmlPullParser.TEXT) {
            ret = parser.getText();
            parser.nextTag();
        }

        return ret;

    }
}
